      SUBROUTINE ISLAND(NNRM,NM,EMX,IJP,NIJP,IJQ,NIJQ,
     1 U,IPSR,W,V,ISU,ISUSP)
C
C  PARAMETERS:
C
C  NNRM = NUMBER OF MATRICES
C
C  NM = THE ORDER OF THE MATRICES
C
C  EMX(NM,NM,NNRM) = E-VALUES; NOT OBSERVED REFLEXIONS ARE ZERO
C
C  IREL: NUMBER OF PHASE-SHIFT RELATIONS
C
C  IPSR: MAXIMUM NUMBER OF PHASE-SHIFT RELATIONS
C
C  U(IPSR,NM): PHASE SHIFT RELATION MATRIX, RANGE MATRIX AFTER SVD
C
C  W(NM): CONTAINS VECTOR W FROM SINGULAR VALUE DECOMPOSITION
C
C  V(NM,NM): NULLSPACE MATRIX
C
C  ISU(NM,NM,NNRM): TABLE OF SUSPECT REFLECTIONS
C
C  IJP CONTAINS INFORMATION ABOUT NIJP INDEPENDENT (=PARENT) REFLECTIONS
C  THE ARRAY IJP(I,*) IS CONFIGURATED AS FOLLOWS
C
C  I=1  IRY = ROW NUMBER
C  I=2  IKO = COLUMN NUMBER
C  I=3  IMA = MATRIX NUMBER
C  I=4  IBE = BEGIN OF RELATIONS OF THIS REFLECTION IN IJQ
C  I=5  IEI = END OF RELATIONS IN IJQ, IEI=0: NO RELATIONS
C  I=6  MK  : PHASE IS RESTRICTED TO 15*(MK-1) (+PI) UNLESS MK=1
C  I=7  ITY : 1: ORIGIN DEFINING REFLECTION
C             2: SYMBOL
C             3: REFLECTION WILL NOT BE REFINED
C
C
C  IJQ CONTAINS INFORMATION ABOUT NIJQ SYMMETRY RELATED REFLECTIONS
C  PMX(I1,J1,M1)=PMX(I2,J2,M2)*(1-2*IT)+IS*PI/12
C
C  THE ARRAY IJQ(I,*) IS CONFIGURED AS FOLLOWS:
C
C  I=1  I1 = ROW NUMBER OF SYMMETRY DEPENDENT REFLECTION
C  I=2  J1 = COLUMN NUMBER OF SYMMETRY DEPENDENT REFLECTION
C  I=3  M1 = MATRIX NUMBER OF SYMMETRY DEPENDENT REFLECTION
C  I=4  I2 = ROW NUMBER OF PARENT REFLECTION
C  I=5  J2 = COLUMN NUMBER OF PARENT REFLECTION
C  I=6  M2 = MATRIX NUMBER OF PARENT REFLECTION
C  I=7  IS = ADDED PHASE
C  I=8  IT = SIGN OF PARENT PHASE
C
C  IS AND IT DEFINE THE PHASE RELATION
C  NOTE: NOT OBSERVED ARE NOT STORED IN IJP AND IJQ
C
      CHARACTER*11 FRNT
      CHARACTER*9  FRNT2
      INTEGER ISU(NM,NM,NNRM)
      REAL*4 U(IPSR,NM),W(NM),V(NM,NM)
C
C  DOUBLE PRECISION AND WORK-SPACE IS NEEDED FOR NAG ROUTINE
C  SINGULAR VALUE DECOMPOSITION FROM NAG WORKS PERFECT!
C
C     REAL*8 U(IPSR,NM),W(NM),V(NM,NM),WORK(1000)
      DIMENSION EMX(NM,NM,NNRM)
      DIMENSION IJP(7,NIJP),IJQ(8,NIJQ)
      COMMON /UNIT/ LN,LO,ISTAN,ISCRA,LNTA,IFMLTN,IFDET
C
      WRITE(FRNT,'(''(1X,'',I2,''F5.2)'')') NM
      WRITE(FRNT2,'(''(1X,'',I2,''I1)'')') NM
C
      EMIN=ISUSP/1000.
      NRDEL=0
      TOL=1.0E-09
C
C  START SEARCH FOR PHASE-SHIFT RELATIONS
C
C  LOOP OVER KARLE-HAUPTMAN MATRICES
C
      DO 1000 IM=1,NNRM
C
C  INITIALIZE PHASE-SHIFT RELATION MATRICES U(*,*) AND IREL
C
      DO 20 J=1,IPSR
      DO 30 K=1,NM
      U(J,K)=0.0
      ISU(K,K,IM)=0
   30 CONTINUE
   20 CONTINUE
      IREL=0
C
C  LOOP OVER INDEPENDENT REFLECTIONS (NIJP)
C
      DO 100 I=1,NIJP
C
C  LOCATE PARENT REFLECTION
C
      IRYPAR=IJP(1,I)
      IKOPAR=IJP(2,I)
      IMAPAR=IJP(3,I)
C
C  CHECK FOR SUFFICIENT HIGH E-VALUE
C
      IF (EMX(IRYPAR,IKOPAR,IMAPAR).GT.EMIN) THEN
C
C  GET INFORMATION ABOUT DEPENDENT REFLECTION FROM IJP(*,NIJP)
C
      IEIPAR=IJP(5,I)
C
C  CHECK FOR EXISTENCE OF DEPENDENT REFLECTIONS
C
      IF (IEIPAR.GT.0) THEN
C
      IBEPAR=IJP(4,I)
C
C  LOOP OVER ALL DEPENDENT REFLECTIONS BELONGING TO PARENT REFLECTION
C
      DO 200 J=IBEPAR,IEIPAR
      IRYDEP=IJQ(1,J)
      IKODEP=IJQ(2,J)
      IMADEP=IJQ(3,J)
C
C  LOOK FOR IDENTICAL MATRIX NUMBER
C
      IF ((IMAPAR.EQ.IM).AND.(IMADEP.EQ.IM)) THEN
C
C  RELATION FOUND BETWEEN DEPENDENT AND INDEPENDENT IN MATRIX IM
C
      IREL=IREL+1
      IF (IREL.GT.IPSR) THEN
      WRITE(LO,*)
     1 ' ISLAND: NUMBER OF PSR EXCEEDS PHYSICAL DIMENSION'
      STOP
      ENDIF
C
C  GET INFORMATION ABOUT DEPENDENT REFLECTION FROM IJQ(*,NIJQ)
C
      ITIDEP=IJQ(8,J)
C
C  CHECK INFORMATION ABOUT PARENT REFLECTION
C
      IF ((IJQ(4,J).NE.IRYPAR).OR.(IJQ(5,J).NE.IKOPAR).OR.
     1 (IJQ(6,J).NE.IMAPAR)) THEN
      WRITE(LO,*) 'ERROR ON ISLAND'
      STOP
      ENDIF
C
C  CONSTRUCT PHASE-SHIFT RELATION:
C  FORM: PHI(IKOPAR) - PHI(IRYPAR) = ITIDEP*( PHI(IKODEP) - PHI(IRYDEP)
C
C     WRITE(LO,*) IREL,IMAPAR,IKOPAR,IRYPAR,IRYDEP,IKODEP
C
      U(IREL,IKOPAR) = U(IREL,IKOPAR) + 1
      U(IREL,IRYPAR) = U(IREL,IRYPAR) - 1
      U(IREL,IRYDEP) = U(IREL,IRYDEP) + (1-2*ITIDEP)
      U(IREL,IKODEP) = U(IREL,IKODEP) - (1-2*ITIDEP)
C
      ENDIF
C
C  CAN RELATION EXIST BETWEEN OTHER DEPENDENTS?
C
      IF (J.LT.IEIPAR) THEN
C
C  LOOP OVER OTHER DEPENDENT REFLECTIONS
C
      DO 300 K=J+1,IEIPAR
      IMADE2=IJQ(3,K)
C
C  LOOK FOR IDENTICAL MATRIX NUMBER
C
      IF ((IMADEP.EQ.IM).AND.(IMADE2.EQ.IM)) THEN
C
C  RELATION FOUND BETWEEN TWO DEPENDENTS IN MATRIX IM
C
      IREL=IREL+1
      IF (IREL.GT.IPSR) THEN
      WRITE(LO,*)
     1 ' ISLAND: NUMBER OF PSR EXCEEDS PHYSICAL DIMENSION'
      STOP
      ENDIF
C
C  GET INFORMATION ABOUT OTHER DEPENDENT REFLECTION FROM IJQ(*,NIJQ)
C
      IRYDE2=IJQ(1,K)
      IKODE2=IJQ(2,K)
      ITIDE2=IJQ(8,K)
C
C  CHECK INFORMATION ABOUT PARENT REFLECTION
C
      IF ((IJQ(4,K).NE.IRYPAR).OR.(IJQ(5,K).NE.IKOPAR).OR.
     1 (IJQ(6,K).NE.IMAPAR)) THEN
      WRITE(LO,*) 'ERROR ON ISLAND'
      STOP
      ENDIF
C
C  CONSTRUCT PHASE-SHIFT RELATION:
C  FORM: PHI(IKODEP) - PHI(IRYDEP) = ITIDE2*( PHI(IKODE2) - PHI(IRYDE2)
C
C     WRITE(LO,*) IREL,IMADEP,IKODEP,IRYDEP,IRYDE2,IKODE2
C
      U(IREL,IKODEP) = U(IREL,IKODEP) + 1
      U(IREL,IRYDEP) = U(IREL,IRYDEP) - 1
      U(IREL,IRYDE2) = U(IREL,IRYDE2) + (1-2*ITIDE2)
      U(IREL,IKODE2) = U(IREL,IKODE2) - (1-2*ITIDE2)
C
      ENDIF
  300 CONTINUE
      ENDIF
  200 CONTINUE
      ENDIF
      ELSE
      NRDEL=NRDEL+1
      ENDIF
  100 CONTINUE
C     WRITE(LO,1) NRDEL
    1 FORMAT(' ',I5,' SHIFT-RELATIONS IGNORED BY E-VALUE')
C
C  PHASE-SHIFT RELATIONS DETERMINED
C
C  DETERMINE NULLSPACE USING SINGULAR VALUE DECOMPOSITION
C
C  IF NUMBER OF RELATIONS IS SMALLER THAN THE ORDER, ADD RELATIONS
C  CONTAINING ONLY ZERO's
C
      IF (IREL.LT.NM) IREL=NM
C
C     WRITE(LO,*)
C     WRITE(LO,FRNT) ((U(IP1,IP2),IP2=1,NM),IP1=1,IREL)
C
      CALL SVDRAG(U,V,IREL,NM,IPSR,NM,TOL)
      CALL SINGVAL(U,V,W,IREL,NM,IPSR,NM,TOL)
      DO 2000 KHRY=2,NM
      JMAX=KHRY-1
      DO 3000 KHKO=1,JMAX
      NIET=0
      DO 4000 IVRY=1,NM
      IF (ABS(W(IVRY)).LT.0.00001) THEN
C
C  NAG AND SVDRAG GIVE V IN A DIFFERENT WAY
C
C     IF (ABS(V(IVRY,KHRY)-V(IVRY,KHKO)).GE.0.00001) NIET=1
      IF (ABS(V(KHRY,IVRY)-V(KHKO,IVRY)).GE.0.00001) NIET=1
      ENDIF
 4000 CONTINUE
      IF (NIET.EQ.0) THEN
      ISU(KHRY,KHKO,IM)=1
      ISU(KHKO,KHRY,IM)=1
      ELSE
      ISU(KHRY,KHKO,IM)=0
      ISU(KHKO,KHRY,IM)=0
      ENDIF
 3000 CONTINUE
 2000 CONTINUE
C     WRITE(LO,*)
C     WRITE(LO,*) 'RELIABLE ELEMENTS:'
C     WRITE(LO,*)
C     WRITE(LO,FRNT2) ((ISU(IP1,IP2,IM),IP2=1,NM),IP1=1,NM)
 1000 CONTINUE
      RETURN
      END
