C        *****************************************************
C        *  FFFFF RRRR   AAA   GGG  M   M EEEEE N   N TTTTT  *
C        *  F     R   R A   A G   G MM MM E     NN  N   T    *
C        *  FFF   RRRR  AAAAA G     M M M EEE   N N N   T    *
C        *  F     R  R  A   A G GGG M   M E     N  NN   T    *
C        *  F     R   R A   A  GGG  M   M EEEEE N   N   T    *
C        *****************************************************
C
C  *******************************************************************
C  Program:  fragment (.f,.for,.fortran)
C  Version:  21/03/1995
C  Language: FORTRAN77
C  Purpose:  generates largest fragment from SEARCH80 output 
C  *******************************************************************
C
C  For information, comments, suggestions, complaints, please contact:
C
C  Rene' de Gelder
C  Laboratorium voor Kristallografie
C  Katholieke Universiteit Nijmegen
C  Toernooiveld 1
C  6525 ED Nijmegen
C  Nederland
C
C  E-mail: U625018 at vm.uci.kun.nl
C  Tel:    080 - 652596
C  Fax:    080 - 553450
C
C  *******************************************************************
C
C  Last revision: RdG 18-10-1995: FORMAT atom names and fragment number
C                                 in ATOMS and LOUT file corrected  
C  Last revision: RdG 21-03-1995: READ LCEL adapted 
C  Last revision: RdG 03-02-1995: If largest cluster is 0: take all peaks
C                                 ATOMS-file with peaksnumbers + 100
C                                 Format clusternumber adapted
C  Last revision: RdG 02-02-1995: No fragments: take largest cluster 
C  Last revision: RdG 31-01-1995: Little error in peak number output solved 
C  Last revision: RdG 30-01-1995: Search80 error causes no error anymore 
C                                 Some nice output on unit 6 
C  Last revision: RdG 27-01-1995: PDB and ATOMS-file now will contain all 
C                     clusters (and all peaks instead of single fragment)
C  Last revision: RdG 10-01-1995: PDB-file as output on unit 9
C  Last revision: RdG 02-01-1995: ATOMS-file with peaknumbers + 100
C
C  *******************************************************************
C
      PROGRAM FRAGMENT
C
C  SELECTS LARGEST FRAGMENT FROM COMPLETE PEAKS LIST OF SEARCH80
C  AND SELECTS ONLY THE HEAVY ATOMS FROM THE COMPLETE PEAK LIST
C
      INTEGER LPK(2000),LFR(2000),LPEAK(2000),IPC(500,500),LFRAG(2000),
     1 LNMFR(2000),LNMFR2(2000),IPOINT(2000),IHEIG(2000),ICLUS(2000)
      REAL X(2000),Y(2000),Z(2000),FI(2000)
      CHARACTER*4 LABEL(2000), DUMMY
      IEIND=0
      MAXFRA=0
C
C  LPEK: COMPLETE PEAK-LIST FILE FROM SEARCH80
C
      LPEK=1
C
C  LFRA: FRAGMENT INFORMATION FILE FROM SEARCH80
C
      LFRA=2
C
C  LPK2: PEAK LIST FILE OF LARGEST FRAGMENT
C
      LPK2=3
C
C  LOUT: PRINTABLE OUTPUT FILE
C
      LOUT=6
C
C  LTYP: FILE CONTAINING STRUCTURE TYPE INFORMATION (EQUAL/HEAVY ATOM)
C
      LTYP=4
C
C  LCEL: FILE CONTAINING CELL INFORMATION 
C
      LCEL=7
C
C  LATM: FILE OF LARGEST FRAGMENT in ATOMS (DIRDIF) format
C
      LATM=8
C
C  LPDB: FILE OF LARGEST FRAGMENT in PDB format
C
      LPDB=9
      WRITE(LOUT,*) 'PROGRAM FRAGMENT'
C
C  READ STRUCTURE TYPE INFORMATION
C
      READ(LTYP,1234) ITYPE
      IF (ITYPE.EQ.1) THEN
      WRITE(LOUT,*) 'HEAVY ATOM STRUCTURE'
      READ(LTYP,1234) INUMH
      WRITE(LOUT,*) INUMH, 'HEAVY ATOMS'
      READ(LTYP,1234) ISCAT
      WRITE(LOUT,*) 'SCATTERING FACTOR TABLE NUMBER ', ISCAT
      ELSE
      WRITE(LOUT,*) 'EQUAL ATOM STRUCTURE'
      INUMH=0
      ISCAT=0
      ENDIF
 1234 FORMAT(I5)
C
C  READ CELL AND PREPARE FRACTIONAL TO CARTESIAN MATRIX
C
      PI = 355. / 113.
      READ(LCEL,*)   AAS, BAS, CAS, ALPHA, BETA, GAMMA
      WRITE(LOUT,*)  AAS, BAS, CAS, ALPHA, BETA, GAMMA
      ALPHA = PI * ALPHA / 180.
      BETA  = PI * BETA  / 180.
      GAMMA = PI * GAMMA / 180.
      A1 = AAS
      A2 = 0
      A3 = 0
      B1 = BAS * COS(GAMMA)
      B2 = BAS * SIN(GAMMA)
      B3 = 0
      C1 = CAS * COS(BETA)
      C2 = CAS * COS(ALPHA)/ (CAS * COS(GAMMA) * COS(BETA) + SIN(GAMMA))
      C3 = CAS * (1 - (COS(BETA))**2 - ((COS(ALPHA))**2) /
     1 (CAS * COS(GAMMA) * COS(BETA) + SIN(GAMMA))**2)**0.5
      V1 = ( B2 * C3 - B3 * C2)
      V2 = (-B1 * C3 + B3 * C1)
      V3 = ( B1 * C2 - B2 * C1)
      V  = A1 * V1 + A2 * V2 + A3 * V3
C     WRITE(*,*) V
C
C  READ COMPLETE PEAK-LIST
C
      IPK=0
  100 CONTINUE 
      IPK=IPK+1                                                         
      READ(LPEK,10,END=200)LABEL(IPK),FI(IPK),X(IPK),Y(IPK),Z(IPK),
     1 IHEIG(IPK),ICLUS(IPK)
      WRITE(LOUT,10)LABEL(IPK),FI(IPK),X(IPK),Y(IPK),Z(IPK),
     1 IHEIG(IPK),ICLUS(IPK)
      IF (IPK.LE.INUMH) FI(IPK)=ISCAT
   10 FORMAT(A4,6X,4F10.5,2I10)                                              
      GOTO 100                          
  200 CONTINUE
C     WRITE(*,*) 'IPK= ',IPK
      IPK=IPK-1
C     WRITE(*,*) 'IPK= ',IPK
C
C  IN CASE HEAVY ATOM STRUCTURE ONLY PUT OUT HEAVY ATOMS
C
      IF (ITYPE.EQ.1) THEN
      DO 4000 M=1,INUMH
      WRITE(LOUT,10)LABEL(M),FI(M),X(M),Y(M),Z(M),IHEIG(M),ICLUS(M)
      WRITE(LPK2,10)LABEL(M),FI(M),X(M),Y(M),Z(M),IHEIG(M),ICLUS(M)
 4000 CONTINUE
C     STOP
      ENDIF
C
C  READ FRAGMENT INFORMATION
C
  250 CONTINUE
      IMAFO=0
      ITEL=0
C
C  READ PEAK NUMBER AND CORRESPONDING FRAGMENT NUMBER
C
      READ(LFRA,20,END=1000) ITYP
      IF(ITYP.EQ.0) GOTO 1000
      IF(ITYP.NE.1) WRITE(*,*) ' READ ERROR'
  300 CONTINUE
      READ(LFRA,20) (LPK(I),I=1,25) 
      READ(LFRA,20) ITYP
      IF(ITYP.NE.2) WRITE(*,*) ' READ ERROR'
      READ(LFRA,20) (LFR(I),I=1,25)
      DO 400 J=1,25
      IF(LPK(J).NE.0) THEN
      ITEL=ITEL+1
      LPEAK(ITEL)=LPK(J)
      LFRAG(ITEL)=LFR(J)
      ELSE
      READ(LFRA,20) ITYP
      IF(ITYP.NE.3) WRITE(*,*) ' READ ERROR'
      GOTO 500
      ENDIF
  400 CONTINUE
      READ(LFRA,20) ITYP
      IF(ITYP.EQ.1) GOTO 300
  500 CONTINUE
      IF(ITYP.NE.3) WRITE(*,*) ' READ ERROR'
C
C  DETERMINE NUMBER OF PEAKS IN EACH FRAGMENT
C
      DO 525 M=1,500
      LNMFR2(M) = 0
  525 CONTINUE
      DO 550 I=1,ITEL
      LNMFR2(LFRAG(I)) = LNMFR2(LFRAG(I)) + 1
      WRITE(LOUT,*) 'ATOM', LPEAK(I),'FRAGMENT', LFRAG(I)
  550 CONTINUE
C
C  READ NUMBER OF PEAKS IN EACH FRAGMENT
C
      IFRA=0
      READ(LFRA,20) (LFR(I),I=1,25)
      DO 600 J=1,25
      IF(LFR(J).NE.0) THEN
      IFRA=IFRA+1
      LNMFR(IFRA)=LFR(J)
      WRITE(LOUT,*) 'FRAGMENT', IFRA, 'NUMBER OF ATOMS MAIN:',
     1 LNMFR(IFRA),'| RDG: ',LNMFR2(IFRA)
      ELSE
      GOTO 700
      ENDIF
  600 CONTINUE
  700 CONTINUE
C
C  DETERMINE LARGEST FRAGMENT
C
      ISOM=0
      DO 800 K=1,IFRA
      ISOM=ISOM+LNMFR2(K)
      IF(LNMFR2(K).GT.MAXFRA) THEN
      MAXFRA=LNMFR2(K)
      LARFRA=K
      IMAFO=1
      ENDIF
  800 CONTINUE
      WRITE(LOUT,*) 'LARGEST FRAGMENT',LARFRA,'WITH',MAXFRA,'PEAKS'
C
C  CHECK IF NUMBER OF PEAKS IS THE SAME AS SUM OF PEAKS IN EACH FRAGMENT
C 
      IF(ISOM.NE.ITEL) THEN
      WRITE(LOUT,*)' ITEL.NE.ISOM', ITEL,ISOM
      ENDIF
C
C  COLLECT PEAKS IN LARGEST FRAGMENT
C
      IF(IMAFO.EQ.1) THEN
      ILFT=0
      DO 900 L=1,ITEL
      IF(LFRAG(L).EQ.LARFRA) THEN
      ILFT=ILFT+1
      IPOINT(ILFT)=LPEAK(L)
      ENDIF
  900 CONTINUE      
C
C  CHECK IF NUMBER OF PEAKS FOUND THE SAME AS SUM OF PEAKS IN LARGEST FRAGMENT
C 
      IF(ILFT.NE.MAXFRA) THEN
      WRITE(LOUT,*)' ILFT.NE.MAXFRA', ILFT, MAXFRA
      ENDIF
      ENDIF
      GOTO 250
 1000 CONTINUE
      IF (ITEL.EQ.0. AND. MAXFRA.EQ.0) THEN
      WRITE(LOUT,*) 'NO FRAGMENTS FOUND: TAKE LARGEST CLUSTER!'
      GOTO 1150
      ENDIF
      IF (ITYPE.NE.1) THEN
      DO 1100 M=1,MAXFRA
      WRITE(LPK2,10)LABEL(IPOINT(M)),FI(IPOINT(M)),X(IPOINT(M)),
     1 Y(IPOINT(M)),Z(IPOINT(M)),IHEIG(IPOINT(M)),ICLUS(IPOINT(M))
      WRITE(LOUT,10)LABEL(IPOINT(M)),FI(IPOINT(M)),X(IPOINT(M)),
     1 Y(IPOINT(M)),Z(IPOINT(M)),IHEIG(IPOINT(M)),ICLUS(IPOINT(M))
 1100 CONTINUE
      ENDIF
 1150 CONTINUE
C  Set number of atoms in clusters to zero
      DO 1200 M=1,500
      LNMFR(M) = 0
 1200 CONTINUE
C  Sort peaks into clusters
C     WRITE(*,*) 'IPK= ',IPK
      DO 1300 M=1,IPK
      ICLUS(M) = ICLUS(M) + 1
      LNMFR(ICLUS(M))=LNMFR(ICLUS(M)) + 1
      IPC(ICLUS(M),LNMFR(ICLUS(M)))=M
 1300 CONTINUE
C  Determine number of clusters and largest cluster
      ICLU = 0
      MAXCLU = 0
      LARCLU = 0
      DO 1400 M=1,100
      IF (LNMFR(M).GT.0.OR.M.EQ.1) THEN
      ICLU = ICLU + 1
      IF(LNMFR(M).GT.MAXCLU) THEN
      MAXCLU=LNMFR(M)
      LARCLU=M
      ENDIF
      ELSE
      GOTO 1400
      ENDIF
 1400 CONTINUE
      IF (ITYPE.NE.1 .AND. MAXFRA.EQ.0) THEN
      IF (LARCLU.NE.1) THEN
      DO 1450 M=1,MAXCLU
      IPOINT(M)=IPC(LARCLU,M)
      WRITE(LPK2,10)LABEL(IPOINT(M)),FI(IPOINT(M)),X(IPOINT(M)),
     1 Y(IPOINT(M)),Z(IPOINT(M)),IHEIG(IPOINT(M)),ICLUS(IPOINT(M))-1
      WRITE(LOUT,10)LABEL(IPOINT(M)),FI(IPOINT(M)),X(IPOINT(M)),
     1 Y(IPOINT(M)),Z(IPOINT(M)),IHEIG(IPOINT(M)),ICLUS(IPOINT(M))-1
 1450 CONTINUE
      ELSE
      WRITE(LOUT,*) 'LARGEST CLUSTER = 0: USE ALL PEAKS!'
      DO 1475 M=1,IPK
      WRITE(LOUT,10)LABEL(M),FI(M),X(M),Y(M),Z(M),
     1 IHEIG(M),ICLUS(M)-1
      WRITE(LPK2,10)LABEL(M),FI(M),X(M),Y(M),Z(M),
     1 IHEIG(M),ICLUS(M)-1
 1475 CONTINUE
      ENDIF
      ENDIF
C  Put out complete ATOMS- and PDB-file, sorted on cluster number
      WRITE(LATM,40)
      WRITE(LOUT,40)
      ITEL = 0
C     WRITE(*,*) 'ICLU= ',ICLU
      DO 1500 M=1,ICLU
      IF(LNMFR(ICLU).NE.0) THEN
      DO 1600 N=1,LNMFR(M)
      ITEL = ITEL + 1
      XF = X(IPC(M,N))
      YF = Y(IPC(M,N))
      ZF = Z(IPC(M,N))
      XC = A1 * XF + B1 * YF + C1 * ZF
      YC = A2 * XF + B2 * YF + C2 * ZF
      ZC = A3 * XF + B3 * YF + C3 * ZF
      IF ( M-1.GT.0 .AND. M-1.LT.10 ) THEN 
      WRITE(LATM,31) IPC(M,N)+100*(M-1),X(IPC(M,N)),
     1 Y(IPC(M,N)),Z(IPC(M,N)),IHEIG(IPC(M,N)),M-1 
      WRITE(LOUT,31) IPC(M,N)+100*(M-1),X(IPC(M,N)),
     1 Y(IPC(M,N)),Z(IPC(M,N)),IHEIG(IPC(M,N)),M-1 
      ENDIF
      IF ( M-1.GE.10 ) THEN 
      WRITE(LATM,30) IPC(M,N)+100*(M-1),X(IPC(M,N)),
     1 Y(IPC(M,N)),Z(IPC(M,N)),IHEIG(IPC(M,N)),M-1 
      WRITE(LOUT,30) IPC(M,N)+100*(M-1),X(IPC(M,N)),
     1 Y(IPC(M,N)),Z(IPC(M,N)),IHEIG(IPC(M,N)),M-1 
      ENDIF
      IF ( M-1.EQ.0 .AND. IPC(M,N).LT.10 )THEN 
      WRITE(LATM,32) IPC(M,N)+100*(M-1),X(IPC(M,N)),
     1 Y(IPC(M,N)),Z(IPC(M,N)),IHEIG(IPC(M,N)),M-1 
      WRITE(LOUT,32) IPC(M,N)+100*(M-1),X(IPC(M,N)),
     1 Y(IPC(M,N)),Z(IPC(M,N)),IHEIG(IPC(M,N)),M-1 
      ENDIF
      IF ( M-1.EQ.0 .AND. (IPC(M,N).LT.100.AND.IPC(M,N).GT.9) )THEN 
      WRITE(LATM,33) IPC(M,N)+100*(M-1),X(IPC(M,N)),
     1 Y(IPC(M,N)),Z(IPC(M,N)),IHEIG(IPC(M,N)),M-1 
      WRITE(LOUT,33) IPC(M,N)+100*(M-1),X(IPC(M,N)),
     1 Y(IPC(M,N)),Z(IPC(M,N)),IHEIG(IPC(M,N)),M-1 
      ENDIF
      IF ( M-1.EQ.0 .AND. IPC(M,N).GE.100 )THEN 
      WRITE(LATM,34) IPC(M,N)+100*(M-1),X(IPC(M,N)),
     1 Y(IPC(M,N)),Z(IPC(M,N)),IHEIG(IPC(M,N)),M-1 
      WRITE(LOUT,34) IPC(M,N)+100*(M-1),X(IPC(M,N)),
     1 Y(IPC(M,N)),Z(IPC(M,N)),IHEIG(IPC(M,N)),M-1 
      ENDIF
      WRITE(LPDB,60) ITEL,M-1,XC,YC,ZC
 1600 CONTINUE
      ENDIF
 1500 CONTINUE
      WRITE(LATM,50)
      WRITE(LOUT,50)
   20 FORMAT(25I3)
   30 FORMAT('ATOM C',I4,3F10.5,'  1.000   3.000',I9,'$',I2)
   31 FORMAT('ATOM C',I3,1X,3F10.5,'  1.000   3.000',I9,'$',I1)
   32 FORMAT('ATOM C',I1,3X,3F10.5,'  1.000   3.000',I9,'$',I1)
   33 FORMAT('ATOM C',I2,2X,3F10.5,'  1.000   3.000',I9,'$',I1)
   34 FORMAT('ATOM C',I3,1X,3F10.5,'  1.000   3.000',I9,'$',I1)
   40 FORMAT('ATOMS CRUNCH')                                              
   50 FORMAT('END')             
   60 FORMAT('ATOM  ',I5,1X,' C',I2,' ','RDG',1X,'      ',3X,3F8.3)
      STOP
      END                                                         
