C  *******************************************************************
C  Program:  thinkc (.f,.for,.fortran)
C  Version:  01/02/1995
C  Language: FORTRAN77
C  Purpose:  automatic generation of input-cards for DETER
C  *******************************************************************
C
C  For information, comments, suggestions, complaints, please contact:
C
C  Rene' de Gelder
C  Laboratorium voor Kristallografie
C  Katholieke Universiteit Nijmegen
C  Toernooiveld 1
C  6525 ED Nijmegen
C  Nederland
C
C  E-mail: U625018 at vm.uci.kun.nl
C  Tel:    080 - 652596
C  Fax:    080 - 553450
C
C  *******************************************************************
C
C  Last revision: RdG 01-02-1995:  Recycle all atoms 
C  Last revision: RdG 01-02-1995:  Recycle only half the number of atoms 
C  Last revision: RdG 31-01-1995:  Recycling implemented 
C  Last revision: RdG 16-01-1995:  If Nr. of Sym.Dep. > 5*NATAU: start 
C  Last revision: RdG 11-01-1995:  Minimum order of KH-matrices = 3
C
C***********************************************************************
C  TTTTT H   H IIIII N   N K   K  CCC                                  *
C    T   H   H   I   NN  N K  K  C   C   -----------------------       *
C    T   HHHHH   I   N N N KKK   C      | LAST EDIT 28-04-97 RAG|      *
C    T   H   H   I   N  NN K  K  C   C   -----------------------       *
C    T   H   H IIIII N   N K   K  CCC    ALL STRUCTURES NOW POSSIBLE!  *
C***********************************************************************
C
C  R.A.G. de Graaff
C  XRAY-DEPARTMENT GORLAEUS LABORATORIES
C  PO BOX 9502, 2300 RA LEIDEN
C  THE NETHERLANDS
C
      PROGRAM THINKC
      CHARACTER*80 DUMMY(15),TEKST
      LOGICAL PAORDE,PAREEK,UPORDE,UPREEK
C
C  THIS PROGRAM GENERATES INPUT FOR DETER 
C
C-----------------------------------------------------------------------
C
C  SYMMETRY FILE: NCENTR (CENTROSYMMETRIC STRUCTURES: 1, NON CENTRO: 0)
C                 NUMBER OF OPERATIONS, NUMBER OF ATOMS IN THE CELL
C                 OPTIONAL: NUMBER OF MATRICES AND ORDER OF THE MATRICES
C
      LSYM=24
C
C  INPUT FILE DETER
C
      LN=25
C
C  OUTPUT FILE DETER
C
      LO=26
C
C  FILE CONTAINING NUMBER OF FASTAN REFLECTIONS IN SINGLE MATRIX
C
      LNUMS=27
C
C  FILE CONTAINING NUMBER OF FASTAN REFLECTIONS IN MULTI MATRIX
C
      LNUMM=28
C
C  CHECK FILE STAGE 1
C
      LS=29
C
C  CHECK FILE STAGE 2
C
      LM=30
C-----------------------------------------------------------------------
C
C  READ NCENTR, NUMBER OF SYMMETRY OPERATIONS, NUMBER OF ATOMS IN THE CELL,
C  NUMBER OF MATRICES AND ORDER OF THE MATRICES (IF SPECIFIED)
C
      READ (LSYM,*) NCENTR
      READ (LSYM,*) NOP
      READ (LSYM,*) NAT
      READ (LSYM,*) NMA
      READ (LSYM,*) IORDE
      READ (LSYM,*) NBG
      READ (LSYM,*) NEI
      READ (LSYM,*) IREC
      READ (LSYM,*) NREEKS
C
C  NUMBER OF ATOMS IN THE ASYMMETRIC UNIT:
C
      NATAU=NAT/NOP
C
C  LOWER LIMIT FOR NUMBER OF FASTAN REFLECTIONS NEEDED
C
C  MULTI-MATRIX: CENTROSYMMETRIC: FASTAN REFLECTIONS NEEDED: 2.5*NATAU
C                NON CENTROSYMMETRIC:                        2.0*NATAU
C
      ILIM1=20
      IF (NCENTR.EQ.1) ILIM2=2.5 * NATAU
      IF (NCENTR.EQ.0) ILIM2=2.0 * NATAU
      IF (NCENTR.EQ.1) ILIM4=5.0 * NATAU
      IF (NCENTR.EQ.0) ILIM4=5.0 * NATAU
C
C  IF NOT SPECIFIED BY USER CALCULATE OPTIMAL ORDER
C
      IF (IORDE.EQ.0) THEN
C
C  OPTIMAL ORDER FOR MATRICES: CENTROSYMMETRIC:     NAT/6
C                              NON CENTROSYMMETRIC: NAT/4
C
      IF (NCENTR.EQ.1) IORDE=NAT/6
      IF (NCENTR.EQ.0) IORDE=NAT/4
C
C  ORDER MUST BE ODD FOR COMMON MINOR STRATEGY
C
      IF((IORDE/2)*2.EQ.IORDE) THEN
      IORDE=IORDE-1
      ENDIF
C
C  AVOID ORDER SMALLER THAN 11 
C
      IORDE = MAX(11,IORDE)
      ENDIF
      IF(NCENTR.EQ.1)THEN
      IOLD=NAT/6
      ELSEIF(NCENTR.EQ.0)THEN
      IOLD=NAT/4
      ENDIF
      IF(IOLD.LT.11)IOLD=11
      IF(IOLD/2*2.EQ.IOLD)IOLD=IOLD-1
C-----------------------------------------------------------------------
C                                                                       
C  INPUT CARD 1: GENERAL PARAMETERS
C
C  NNRM:   NUMBER OF LINKED MATRICES TO BE BUILT                        
C          NOTE ONLY ONE ORDER FOR ALL MATRICES IN THIS ROUTINE         
C          THIS MUST BE CHANGED IF DIFFERENT ORDERS ARE REQUIRED        
C          SEE COMMENT SUBR DET1                                        
C  INMX:   ORDER OF THE MATRICES                                
C          USE INMX=0 IF ILEES=1 OR 2
C  NINMX:  ORDER OF THE INNER CORES                             
C          NINMX<0: AN OUTER RING IS CONSTRUCTED
C  ILEES:  0: BUILD MATRICES                                            
C          1: READ MATRICES FROM DISK (FILE 23), NNRM IS IGNORED        
C             NO OPTIMIZATION OF MATRIX IN OPMAT VERMIN
C          2: READ MATRICES FROM DISK (IDEM)
C             MATRIX IS OPTIMIZED IN OPMAT VERMIN                
C  ISAVE:  #0: SAVE MATRICES ON FILE IFDET                       
C  ICON:   1: CONSTRUCTION OF MATRIX USING METHOD DE GELDER
C          2: CONSTRUCTION OF MATRIX USING METHOD VERMIN
C          3: CONSTRUCTION OF MATRIX USING METHOD DE GELDER
C             FOLLOWED BY METHOD VERMIN
C  INMAT:  NUMBER OF RANDOM MATRICES TO BE OPTIMIZED
C  NCENTR: 0: NON CENTROSYMMETRIC STRUCTURE
C          1: CENTROSYMMETRIC STRUCTURE
C
C  INPUT CARD 2: HASHTABLE PARAMETERS AND CENTROSYMMETRIC PARAMETERS
C
C  NHAS:   THE LENGTH OF THE TABLE
C  IHAS:   HASHVALUE
C  IT:     THE NUMBER OF TESTS TO FIND A BETTER VALUE FOR NHAS
C  ITEST:  THE NUMBER OF TESTS MADE BY TESTHAS
C  NC:     NUMBER OF CHROMOSOMES (DETERMINED BY NSIGR)
C  ISET:   TAKE ISET-TH BEST SET FOR FOURIER CALCULATION (NCENTR=1)
C  IDIAG:  DIAGONAL INCREASE VALUE
C  IKRIK:  NUMBER OF TIMES THE DIAGONAL IS INCREASED BY DIAG
C
C  INPUT CARD 3: CONSTRUCTION PARAMETERS METHOD DE GELDER
C
C  IBK:    MAXIMUM DIMENSION OF TOP ROW MATRIX IN OPMAT2      
C  IBLOK:  NUMBER OF BLOCKS OPTIMIZED IN SUBR OPMATG             
C  INMBK:  =0: TAKE BEST BLOCK FOR CONSTRUCTION KH-MATRIX
C          =N: TAKE N-TH BLOCK FOR CONSTRUCTION
C  NDIS:   =1: DETERMINE DISTRIBUTION OF STRONG REFLECTIONS OVER R**3
C              AND SORT STR ON RECIPROCAL VECTOR LENGTH
C          =0: LEAVE ORIGINAL CONVERGENCE-SEQUENCE UNCHANGED
C  NRL:    =0: CALCULATED UPPER LIMIT FOR 1/D**2 FOR EXPANDING 
C              STRONG REFLECTIONS
C          =N: UPPER LIMIT = N/100 FOR EXPANSION OF STRONG 
C              REFLECTIONS
C  ICRI |
C  ICRA |  DEFINE CRITERION IN OPMAT2: |||E|-ICRA|**ICRI-ICRB|
C  ICRB |  
C
C  INPUT CARD 4: MORE CONSTRUCTION PARAMETERS
C
C  NSTR:   THE  NSTR STRONGEST REFLEXIONS IN REF ARE CONSIDERED STRONG
C          THESE REFLEXIONS ARE USED TO CONSTRUCT THE TOP ROW OF THE
C          MATRIX(CES)
C  MIR:    1: SIR OR MIR PHASES ARE PUT IN, BUILDING IS DIFFERENT THEN
C          0: NORMAL DETER RUN
C  NBEGIN: SEE NEIND
C  NEIND:  PHASES ARE CALCULATED FROM TRIAL NBEGIN UP TO AND INCLUDING
C          TRIAL NEIND; DEFAULTS NBEGIN=1 AND NEIND=INMAT
C  NREFTY: 0: KNOWN PHASES ARE NOT REFINED
C          1: KNOWN PHASES ARE REFINED
C  ISUSP:  MINIMUM |E|-VALUE FOR VALID PHASE-SHIFT RELATION (*100)
C          0: NO SEARCH FOR SUSPECT REFLECTIONS
C  NHKLF:  1: INSTEAD OF NORMAL FILE 15 A FILE CONTAINING H,K,L E(H)
C          PHASE AND RELIABILITY IS READ (FREE FORMAT). (NO 25)
C          <>1 SKIP CARD 6
C  NOTF:   >0: FORBIDDEN REFLECTIONS ARE CONSIDERED USEFUL IN MATRIX
C          0: WE RATE FORBIDDEN REFLECTIONS AS NOT OBSERVED
C
C  INPUT CARD 5: MAXIMIZATION PARAMETERS
C
C  IOPT:   0: PROGRAM STOPS AFTER CONSTRUCTION OF MATRIX                
C          1: FOM IS CALCULATED USING START PHASES
C          2: MAXIMIZE DETERMINANTS, THEN AS 1                   
C  ISIG:   0: USE SIGMA2 RELATIONS TO OBTAIN START                      
C          1: USE RANDOM PHASES TO OBTAIN START
C          <>0 OR <>1: NO CHANGE IN PHASES CALCULATED IN TEOFAS
C  NSIGR:  THE N MOST APPEARING REFLECTIONS WILL BE ASSIGNED
C          RANDOM PHASE-VALUES (ONLY VALID WHEN ISIG=0)
C          CENTROSYMMETRIC STRUCTURES: NSIGR MOST APPEARING REFLECTIONS
C          WILL BE PERMUTED
C  IFAS:   ALLOWS KNOWN PHASES TO BE PUT IN                             
C          >0: INPUT FILE TO FASTAN IS READ (FILE 22)                   
C          THE KNOWN PHASES ARE NOT REFINED                      
C  IEIGEN: >0 NOT THE PHASES IN THE MATRIX ARE PUT OUT ON FILE 24       
C          BUT THE PHASES OBTAINED BY CONSTRUCTING THE MATRIX
C          FROM THE EIGENVECTORS FOR WHICH THE EIGENVALUE IS LARGER
C          THAN THE LARGEST EIGENVALUE DIVIDED BY IEIGEN
C          SEE SUBROUTINE WRFOUR         
C  NATOM:  =0: NO CALCULATED PHASES                              
C              OTHERWISE NATOM ATOMIC POSITIONS MULT*ATOM NUMBER,X,Y,Z  
C  IELIM   =0: ALL INDEPENDENT REFLEXIONS ARE REFINED
C              ELSE E'S WITH E<IELIM/100 ARE SET TO ZERO
C  IOP     =0: PHASES DETERMINED BY NEW METHOD
C  IOP     =1: PHASES DETERMINED FOLLOWING DE GRAAFF, VERMIN
C  IOP     =2: COMBINATION OF OPTIONS 0 AND 1
C  
C  INPUT CARD 6: OUTPUT PARAMETERS
C
C  IPRMX:  #0: PRINT MATRICES                                    
C  IPRIJP: #0: PRINT RELATIONS BETWEEN MATRIX ELEMENTS           
C  IPRIJZ: #0: PRINT MULTAN REFLEXIONS IN THE MATRICES           
C  IPRSTR: #0: PRINT STRONG REFLECTIONS
C  ITRIP:  #0: CALCULATE AND PRINT THREE-PHASE INVARIANTS  
C-----------------------------------------------------------------------
C
C  READ INPUT CARDS
C
      READ(LN,123)DUMMY(1)
      READ(LN,123)DUMMY(2)
C
C  TEXT CARD
C
      READ(LN,123)TEKST
      READ(LN,123)DUMMY(3)
      READ(LN,123)DUMMY(4)
C
C  CARD 1
C
      READ(LN,2)NNRM,INMX,NINMX,ILEES,ISAVE,ICON,INMAT,NDUMMY                
      READ(LN,123)DUMMY(5)
      READ(LN,123)DUMMY(6)
C
C  CARD 2
C
      READ(LN,2)NHAS,IHAS,IT,ITEST,NC,ISET,IDIAG,IKRIK
      READ(LN,123)DUMMY(7)
      READ(LN,123)DUMMY(8)
C
C  CARD 3
C
      READ(LN,2)IBK,IBLOK,INMBK,NDIS,NRL,ICRI,ICRA,ICRB                
      READ(LN,123)DUMMY(9)
      READ(LN,123)DUMMY(10)
C
C  CARD 4
C
      READ(LN,2)NSTR,MIR,NBEGIN,NEIND,NREFTY,ISUSP,NHKLF,NOTF
      READ(LN,123)DUMMY(11)
      READ(LN,123)DUMMY(12)
C
C  CARD 5
C
      READ(LN,2)IOPT,ISIG,NSIGR,IFAS,IEIGEN,NATOM,IELIM,IOP                
      READ(LN,123)DUMMY(13)
      READ(LN,123)DUMMY(14)
C
C  CARD 6
C
      READ(LN,2)IPRMX,IPRIJP,IPRIJZ,IPRSTR,ITRIP
      READ(LN,123)DUMMY(15)
    2 FORMAT(8I10)                                                      
  123 FORMAT(A80)
C-----------------------------------------------------------------------
C                                                                       
C  DEFAULTS CARD 1                                                            
C  NNRM=3 unless Inmx=large,ILEES=0,ISAVE=1,ICON=1,INMAT=100
C
      IF (NNRM.EQ.0)  THEN
      IF (INMX.GT.90.OR.IORDE.GT.90) THEN
	NNRM=1
	ELSE
	NNRM=3
	ENDIF
	ENDIF
C     IF (INMX.EQ.0)  INMX=21
      IF (NINMX.EQ.0) NINMX=INMX
      IF (ISAVE.EQ.0) ISAVE=1
      IF (ICON.EQ.0)  ICON=1   
      IF (NEI.NE.0)   INMAT=NEI
      IF (INMAT.EQ.0) INMAT=100
C
C  DEFAULTS CARD 2
C  NHAS=1.3*NTOT+1,IHAS=127,IT=1,ITEST=0,ISET=1,IDIAG=110
C
      IF (IT.EQ.0)    IT=1    
      IF (NHAS.EQ.0)  IT=20  
      IF (IHAS.EQ.0)  IHAS=127
      IF (IDIAG.EQ.0) IDIAG=110
      IF ( (NBG.NE.0) .AND. (ISET.EQ.0) )  ISET=NBG
      IF (ISET.EQ.0)  ISET=1
C
C  DEFAULTS CARD 3
C  IBK=0,IBLOK=1,INMBK=0,NDIS=0,NRL=150,ICRI=2,ICRA=0,ICRB=0
C  DEFAULT IBK IS SUBSTITUTED IN DETER
C
C     IF (IBK.EQ.0)       IBK=800
      IF (IBLOK.EQ.0)     IBLOK=1
      IF (INMBK.GT.IBLOK) INMBK=0
      IF (NRL.EQ.0)       NRL=150
      IF (ICRI.EQ.0)      ICRI=2
C
C  DEFAULTS CARD 4
C  NSTR=3*IBK,MIR=0,NBEGIN=1,NEIND=INMAT,NREFTY=0,ISUSP=0
C  DEFAULT NSTR IS SUBSTITUTED IN DETER
C
      IF ( (NBG.NE.0) .AND. (NBEGIN.EQ.0) )  NBEGIN=NBG
      IF (NBEGIN.EQ.0) NBEGIN=1
      IF (NEIND.EQ.0)  NEIND=NBEGIN                                       
C
C  DEFAULTS CARD 5
C  IOPT=0,ISIG=0,NSIGR=10,IFAS=0,IEIGEN=0,NATOM=0,IELIM=0,IOP=0
C
      IF (NSIGR.EQ.0)  NSIGR=10                                            
      IF (NCENTR.EQ.1) IOP=3
      ELIM=IELIM/100.0
C
C  DEFAULTS CARD 6                                
C  IPRMX=0,IPRIJP=0,IPRIJZ=0,IPRSTR=0,ITRIP=0      
C-----------------------------------------------------------------------
C
C  READ NUMBER OF FASTAN REFLECTIONS
C
      READ(LNUMS,321)IFASS, NIJPS
      READ(LNUMM,321)IFASM, NIJPM 
  321 FORMAT(2I10)
C-----------------------------------------------------------------------
C
C  SITUATION 1: FIRST RUN OF MULTI MATRIX; IFASM=0
C     
      IF(IFASM.EQ.0) THEN
      IF (NMA.EQ.0) THEN
	IF(INMX.LE.90.AND.IORDE.LE.90)THEN
      NNRM=3
	ELSE
	NNRM=1
	ENDIF
      ELSE
      NNRM=NMA
      ENDIF
      INMX=IORDE
      NINMX=INMX
      ILEES=0
      ISAVE=1
      IOPT=0
      IFAS=0
C      IOP=1
      WRITE(LM,124)0
      ENDIF
C
C  SITUATION 2: MATRICES CONSTRUCTED CONTAIN NOT ENOUGH FASTAN REFLECTIONS
C               IF NUMBER OF MATRICES IS SPECIFIED BY USER (NMA.NE.0) THIS
C               STAGE WILL BE SKIPPED
C     
      IF((IFASM.NE.0).AND.(IFASM.LT.ILIM2.AND.NIJPM.LT.ILIM4).AND.
     1 (NMA.EQ.0)) THEN
C     NREEKS=-0.5+INT(SQRT(0.25+2*NNRM)+0.5)
      III=(IORDE-IOLD)/2
      PAORDE=III/4*4.EQ.III
      PAREEK=NREEKS/2*2.LT.NREEKS
      UPREEK=(PAREEK.AND.PAORDE).OR.(.NOT.PAORDE.AND..NOT.PAREEK)
      UPORDE=(PAREEK.AND..NOT.PAORDE).OR.(.NOT.PAREEK.AND.PAORDE)
      UPREEK=UPREEK.OR.(IORDE.GE.50.OR.IORDE.GE.NAT/3-2)
      UPORDE=UPORDE.AND.(IORDE.LT.50.AND.IORDE.LT.NAT/3-2)
      IF(UPORDE)IORDE=IORDE+2
      IF(UPREEK)NREEKS=NREEKS+1
C     write(6,*)IORDE,nreeks,ifasm
      NNRM=0
      DO 100 I=1,NREEKS
      NNRM=NNRM+I
  100 CONTINUE
      INMX=IORDE
      NINMX=INMX
      ILEES=0
      ISAVE=1
      IOPT=0
      IFAS=0
C      IOP=1
      REWIND LSYM
      WRITE(LSYM,*) NCENTR
      WRITE(LSYM,*) NOP
      WRITE(LSYM,*) NAT
      WRITE(LSYM,*) NMA
      WRITE(LSYM,*) IORDE
      WRITE(LSYM,*) NBG
      WRITE(LSYM,*) NEI
      WRITE(LSYM,*) IREC
      WRITE(LSYM,*) NREEKS
      WRITE(LM,124)0
      ENDIF
C
C  SITUATION 4: CYCLE 
C               
      IF(IOPT.EQ.2) THEN
      ILEES=1
      ISAVE=0
      IOPT=2
      IFAS=IFASS
      IF(IFAS.EQ.0) THEN
      ISIG=1
      NBEGIN=NBEGIN+1
      NEIND=NBEGIN
      ISET=ISET+1
      IF(IREC.EQ.1) THEN
      NATOM=NATAU
      ENDIF
      IF(NBEGIN.LE.INMAT) THEN
      WRITE(LM,124)1
      ELSE
      WRITE(LM,124)2
      ENDIF
      ELSE
      NBEGIN=1
      NEIND=NBEGIN
      WRITE(LM,124)1
      ENDIF
C      IOP=1
      ENDIF
C
C  SITUATION 3: MATRICES CONSTRUCTED CONTAIN ENOUGH FASTAN REFLECTIONS
C               TO START CYCLE
C
      IF(((IFASM.GE.ILIM2.OR.NIJPM.GE.ILIM4).OR.
     1  (NMA.NE.0.AND.IFASM.NE.0)).AND.(IOPT.NE.2)) THEN
      ILEES=1
      ISAVE=0
      IOPT=2
      IFAS=IFASS
      IF(IFAS.EQ.0) THEN
      ISIG=1
C     NBEGIN=1
      NEIND=NBEGIN
      ELSE
      NBEGIN=1
      NEIND=NBEGIN
      ENDIF
C      IOP=1
      WRITE(LM,124)0
      ENDIF
C-----------------------------------------------------------------------
C
C  WRITE INPUT CARDS
C
      WRITE(LO,123)DUMMY(1)
      WRITE(LO,123)DUMMY(2)
C
C  TEXT CARD
C
      WRITE(LO,123)TEKST
      WRITE(LO,123)DUMMY(3)
      WRITE(LO,123)DUMMY(4)
C
C  CARD 1
C
      WRITE(LO,2)NNRM,INMX,NINMX,ILEES,ISAVE,ICON,INMAT,NCENTR                
      WRITE(LO,123)DUMMY(5)
      WRITE(LO,123)DUMMY(6)
C
C  CARD 2
C
      WRITE(LO,2)NHAS,IHAS,IT,ITEST,NC,ISET,IDIAG,IKRIK
      WRITE(LO,123)DUMMY(7)
      WRITE(LO,123)DUMMY(8)
C
C  CARD 3
C
      WRITE(LO,2)IBK,IBLOK,INMBK,NDIS,NRL,ICRI,ICRA,ICRB                
      WRITE(LO,123)DUMMY(9)
      WRITE(LO,123)DUMMY(10)
C
C  CARD 4
C
      WRITE(LO,2)NSTR,MIR,NBEGIN,NEIND,NREFTY,ISUSP,NHKLF,NOTF
      WRITE(LO,123)DUMMY(11)
      WRITE(LO,123)DUMMY(12)
C
C  CARD 5
C
      WRITE(LO,2)IOPT,ISIG,NSIGR,IFAS,IEIGEN,NATOM,IELIM,IOP                
      WRITE(LO,123)DUMMY(13)
      WRITE(LO,123)DUMMY(14)
C
C  CARD 6
C
      WRITE(LO,2)IPRMX,IPRIJP,IPRIJZ,IPRSTR,ITRIP
      WRITE(LO,123)DUMMY(15)
  124 FORMAT(I1)
      END
